#include "fritzlistener.h" 
#include "setup.h"
#include <vdr/skins.h>
#include <vdr/device.h>
#include <vdr/remote.h>
#include <list>
#include <vector>

cFritzListener::cFritzListener(std::string &hostname, int port, cFritzFonbuch *fonbuch)
:cTcpClient(hostname, port), cThread("cFritzListener")
{
	this->fonbuch = fonbuch;
	message = NULL;
}

cFritzListener::~cFritzListener()
{
}

void cFritzListener::Action() {
	std::string data;
	std::list<int> connIdList;
	int displayedConnId = -1;
	bool muted  = false;
	bool paused = false;
	while (true) {
		try {
			while (true) {
				dsyslog("FritzBox: Waiting for a message.");
				data.clear();
				*this >> data;
				dsyslog("FritzBox: Got message %s.", data.c_str());

				// parse message
				size_t pos[7];
				memset(pos, std::string::npos, 7);
				pos[0] = 0;
				for (int i=1; i<7; i++) {
					pos[i] = data.find(";", pos[i-1]+1);
					if (pos[i] == std::string::npos)
						break;
				}
				std::string date    = data.substr(pos[0], pos[1]-pos[0]);
				std::string type    = data.substr(pos[1]+1, pos[2]-pos[1]-1);
				int connId = atoi(data.substr(pos[2]+1, pos[3]-pos[2]-1).c_str());
				std::string partA   = pos[4] == std::string::npos ? "" : data.substr(pos[3]+1, pos[4]-pos[3]-1);
				std::string partB   = pos[5] == std::string::npos ? "" : data.substr(pos[4]+1, pos[5]-pos[4]-1);
				std::string partC   = pos[6] == std::string::npos ? "" : data.substr(pos[5]+1, pos[6]-pos[5]-1);

				if (type.compare("CALL") == 0) {
					// partA => box port
					// partB => caller Id
					// partC => called Id
					// partD => medium (POTS, SIP[1-9], ISDN, ...)
				} else if (type.compare("RING") == 0) {
					// partA => caller Id
					// partB => called Id
					// partC => medium (POTS, SIP[1-9], ISDN, ...)
					
					// apply MSN-filter if enabled
					bool notify = fritzboxConfig.msn.size() ? false : true;
					for (std::vector<std::string>::iterator it = fritzboxConfig.msn.begin(); it < fritzboxConfig.msn.end(); it++){
						if (partB.compare(*it) == 0 )
							notify = true;
					} 
					if (notify) {
						connIdList.push_back(connId);
						if (fritzboxConfig.muteOnCall && !cDevice::PrimaryDevice()->IsMute()) {
							cDevice::PrimaryDevice()->ToggleMute();
							muted = true;
						}
						//					if (cControl::c) {
						//						cRemote::Put(kPause);
						//						paused = true;
						//					}
						if (partC.find("SIP") != std::string::npos)
							partC.replace(0, 3, "VoIP ");
						if (partC.find("ISDN") != std::string::npos) {
							partC += " ";
							partC += partB;	
						}
						if (partC.find("POTS") != std::string::npos)
							partC = tr("POTS");
						if (fritzboxConfig.showNumber) {
							// save the message into "message", MainThreadHook will take care of it
							displayedConnId = connId;
							if (partA.size() == 0)
								asprintf(&message, "%s [%s]", tr("Call from unknown extension"), partC.c_str());
							else
								asprintf(&message, "%s %s [%s]", tr("Call from"), (fonbuch->ResolveToName(partA)).c_str(), partC.c_str());	
						}
					}
				} else if (type.compare("CONNECT") == 0) {
					// partA => box port
					// partB => Id
					
					// stop call notification
					if (displayedConnId == connId) {
						free(message);
					    message = NULL;
					    displayedConnId = -1;
					}
				} else if (type.compare("DISCONNECT") == 0) {
					// partA => call duration
					
					// stop call notification
					if (displayedConnId == connId) {
						free(message);
					    message = NULL;
					    displayedConnId = -1;
					}
					// remove current connection from list
					connIdList.remove(connId);
					// unmute, if applicable
					if (connIdList.empty() && muted && cDevice::PrimaryDevice()->IsMute()) {
						cDevice::PrimaryDevice()->ToggleMute();
						muted = false;
					}
//					if (paused && cDevice::PrimaryDevice()->Replaying()) {
//						cDevice::PrimaryDevice()->Play();
//						//cRemote::Put(kPlay);
//						paused = false;
//					}
				} else {
					esyslog("FritzBox: Unknown message type: %s.", type.c_str());
				}
			}
		} catch (cTcpException te) {
			dsyslog("fritzlistener.c: Exception - %s", te.what());
		}
		sleep(RETRY_DELAY); // delay the retry
	}
}
