#include "setup.h"
#include <vdr/menuitems.h>

sFritzboxConfig fritzboxConfig;

const char *cMenuSetupFritzbox::StoreMsn(){
	std::vector<std::string>::iterator it;
	std::string *msnLine = new std::string();
	
	
	for (it = fritzboxConfig.msn.begin(); it < fritzboxConfig.msn.end(); it++){
		*msnLine += *it;
		*msnLine += ' ';
	}
	return msnLine->c_str();
}

void cMenuSetupFritzbox::Setup(void) {
	// save current postion
	int current = Current();
	// clear entries, if any
	Clear();
	// build up setup menu
	Add(new cMenuEditStrItem (tr("Fritz!Box URL"),                  		url,           			MaxFileName, tr(FileNameChars)));
	Add(new cMenuEditStrItem (tr("Password"),                       		password, 	   			MaxFileName, tr(FileNameChars)));
	Add(new cMenuEditBoolItem(tr("Mute on call"),                   		&muteOnCall,   			tr("no"), 	 tr("yes")));
	Add(new cMenuEditBoolItem(tr("Show incoming calls"),            		&showNumber,   			tr("no"), 	 tr("yes")));
	Add(new cMenuEditBoolItem(tr("Detailed call lists"),                    &showNumberInCallList, 	tr("no"), 	 tr("yes")));
	Add(new cMenuEditBoolItem(tr("Hide main menu entry"), 		    		&hideMainMenu, 			tr("no"), 	 tr("yes")));
	Add(new cMenuEditBoolItem(tr("Restrict monitor to certain extensions"), &msnFilter,    			tr("no"), 	 tr("yes")));
	if (msnFilter) {
		Add(new cMenuEditIntItem (tr("Number of monitored extensions"), &msnCount, 1, MAX_MSN_COUNT));
		for (int p = 0; p < msnCount; p++)
			Add(new cMenuEditStrItem(tr("Extension"), msn[p], MaxFileName, "1234567890"));	
	}	
	// restore current position
	SetCurrent(Get(current));
	// refresh display
	Display();
}

eOSState cMenuSetupFritzbox::ProcessKey(eKeys Key) {

	eOSState state = cMenuSetupPage::ProcessKey(Key);

	if (Key != kNone) {
		if (msnFilter != msnFilterBefore) {
			// deactivate MSN Filter
			if (msnFilter == 0) {
				msnCount = 0;
			} else {
				msnCount = 1;
			}
			msnFilterBefore = msnFilter;
		}
		if (msnCount != msnCountBefore) {
			// add new MSN elements
			if (msnCount > msnCountBefore) {
				for (int i=msnCountBefore; i<msnCount; i++) {
					msn[i] = (char *) malloc(MaxFileName * sizeof(char));
					msn[i][0] = 0;
				}
			} 
			// remove existing MSN elements			
			else {				
				for (int i=msnCountBefore; i>msnCount; i--)
					free(msn[i-1]);
			}
			Setup();
			msnCountBefore = msnCount;
		}
	}
	return state;
}

void cMenuSetupFritzbox::Store(void) {
	fritzboxConfig.url            		= url;
	int i = 0;
	// only store the password if it was changed
	while (password[i]) {
		if (password[i] != '*') {
			fritzboxConfig.password       		= password;		
			break;
		}
		i++;
	}
	fritzboxConfig.muteOnCall     		= muteOnCall;
	fritzboxConfig.showNumber     		= showNumber;
	fritzboxConfig.showNumberInCallList = showNumberInCallList;
	fritzboxConfig.hideMainMenu   		= hideMainMenu;
	fritzboxConfig.msn.clear();
	for (int i=0; i < msnCount; i++) {
		std::string s = msn[i];
		fritzboxConfig.msn.push_back(s);
	}
	
	SetupStore("Url",          			url);
	SetupStore("Password",     			fritzboxConfig.password.c_str());
	SetupStore("MuteOnCall",   			muteOnCall);
	SetupStore("ShowNumber",   			showNumber);
	SetupStore("ShowNumberInCallList", 	showNumberInCallList);
	SetupStore("HideMainMenu", 			hideMainMenu);
	SetupStore("MsnList",      			StoreMsn());
}

cMenuSetupFritzbox::cMenuSetupFritzbox()	
{
	// copy setup to temporary parameters
	msn = (char **) malloc(MAX_MSN_COUNT * sizeof(char *));
	url             	 = strdup(fritzboxConfig.url.c_str());
	password        	 = strdup(fritzboxConfig.password.c_str());
	// the original password is not visible in the setup page
	// every single character of the password is displayed as "*"
	for (unsigned int i = 0; i < fritzboxConfig.password.length(); i++) {
		password[i] = '*';
	}
	muteOnCall      	 = fritzboxConfig.muteOnCall;
	showNumber      	 = fritzboxConfig.showNumber;
	showNumberInCallList = fritzboxConfig.showNumberInCallList;
	hideMainMenu    	 = fritzboxConfig.hideMainMenu;
	msnCount        	 = fritzboxConfig.msn.size();
	msnCountBefore  	 = msnCount; // needed for menu refresh
	msnFilter       	 = fritzboxConfig.msn.empty() ? 0 : 1;
	msnFilterBefore 	 = msnFilter;
	
	int p = 0;
	for(std::vector<std::string>::iterator itStr = fritzboxConfig.msn.begin(); itStr < fritzboxConfig.msn.end(); itStr++)
		asprintf(&msn[p++], "%s", itStr->c_str());
	// build up menu entries
	Setup();
}

cMenuSetupFritzbox::~cMenuSetupFritzbox()
{
	// free up malloced space from constructor
	free(url);
	free(password);
	for (int i=0; i<msnCount; i++)
		free(msn[i]);
	free(msn);
}

sFritzboxConfig::sFritzboxConfig() {
	url             		= "fritz.box";
	password        		= "your password";
	muteOnCall      		= 0;
	showNumber      		= 1;
	showNumberInCallList    = 0;
	hideMainMenu    		= 0;
}

bool sFritzboxConfig::SetupParseMsn(const char *value){
	std::string currentMsn;
	unsigned int pos = 0;
	// walk through the complete value-line
	while (value[pos] != 0) {
		currentMsn.clear(); 
		// stop at each <space> or EOL
		while (value[pos] != ' ' && value[pos] != 0) {
			currentMsn += value[pos];
			pos++;
		}
		msn.push_back(currentMsn);
		// at a <space> we have to advance to the next MSN
		if (value[pos] != 0)
			pos++;
	}
	return true;
}

bool sFritzboxConfig::SetupParse(const char *name, const char *value) {
	if      (!strcasecmp(name, "Url"))          		url          		 = value;
	else if (!strcasecmp(name, "Password"))     		password     		 = value;
	else if (!strcasecmp(name, "MuteOnCall"))   		muteOnCall   		 = atoi(value);
	else if (!strcasecmp(name, "ShowNumber"))   		showNumber   		 = atoi(value);
	else if (!strcasecmp(name, "ShowNumberInCallList")) showNumberInCallList = atoi(value);
	else if (!strcasecmp(name, "HideMainMenu")) 		hideMainMenu 		 = atoi(value);
	else if (!strcasecmp(name, "MsnList"))      		return SetupParseMsn(value);
	else return false;
	return true;
}
